#include <stdlib.h>
#include <math.h>
#include <mem.h>
#include <stdio.h>
#include <string.h>

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr
#define MIN(a,b) ((a<b)?a:b)
#define MAX(a,b) ((a>b)?a:b)

#include "..\defs.h"
#include "..\color.h"
#include "..\picture.h"
#include "..\fft2d.h"

fftdata2D *CopyFFTdata(fftdata2D *s)
{
    int y;
    fftdata2D *d;

    d=new fftdata2D;
    if (!d)
       return 0;
    d->sx=s->sx;
    d->sy=s->sy;
    d->nn1=s->nn1;
    d->nn2=s->nn2;
    d->isign=s->isign;
    d->datamax=s->datamax;

    d->data=new float *[d->nn2];
    if (!d->data)
      {
       delete d;
       return 0;
      }
    for( y=0;y<d->nn2;y++ )
         if (d->data[y]=new float[d->nn1<<1])
            memcpy(d->data[y],s->data[y],sizeof(float)*2*d->nn1);
         else break;

    if (y<d->nn2)
      {
       for( y--;y>=0;y-- )
            delete d->data[y];
       delete d->data;
       d->data=0;
       delete d;
       return 0;
      }

    return d;
}

picture8 *BuildFFTimage(fftdata2D *d)
{
    int x,y,i,sx,sy;
    float *f,z,sq,s;
    picture8 *p;
    byteRGB rgb;
    unsigned char *c1;

    sq=(float)d->nn1*d->nn2;
    p=new picture8;
    if (!p)
       return 0;

    if (d->isign==-1)
       if (p->CreatePicture(d->nn1,d->nn2,8,256))
          ;
       else {
             delete p;
             return 0;
            }
    else if (p->CreatePicture(d->sx,d->sy,8,256))
            ;
         else {
               delete p;
               return 0;
              }

    for( i=0;i<256;i++ )
         {
          rgb.r=i; rgb.g=i; rgb.b=i;
          p->SetPalette(i,1,&rgb);
         }
    if (d->isign==-1)
       {
        sx=p->sx;
        sy=p->sy;
        s=255.0/log10(1.0+d->datamax);
        for( y=0;y<sy;y++ )
          {
           f=d->data[y];
           c1=p->buf[y];
           for( x=0;x<sx;x++ )
              {
               z=sqrt((*f)*(*f)+(*(f+1))*(*(f+1)));
               z=s*log10(1.0+z);
               i=(z>255.0?255:(int)z);
               *c1=i;
               c1++;
               f+=2;
              }
          }
       }
    else {
          sx=p->sx;
          sy=p->sy;
          for( y=0;y<sy;y++ )
            {
             f=d->data[y];
             c1=p->buf[y];
             for( x=0;x<sx;x++ )
                {
                 z=*f/sq;
                 i=(z>255.0?255:(int)z);
                 *c1=i;
                 f+=2; c1++;
                }
            }
         }
    return p;
}

void do_anim(char *file,int step)
{
    fftdata2D data,*d;
    int a,x,y,n,dist,cx,cy,t=1,diag;
    float *f;
    picture8 *p8;
    char str[256],*c;

      if (data.LoadFFTdata(file))
         {
           c=strrchr(file,'.');
           if (c)
              *c=0;
           cx=data.nn1/2-1;
           cy=data.nn2/2-1;
//           diag=(int)sqrt((long)data.nn1*data.nn1 + (long)data.nn2*data.nn2)/2;
           diag=cx+1;
           for( a=1;a<diag;a+=step )
             if (d=CopyFFTdata(&data))
             {
              printf("Building frame %i ...\n",t);
//              d->AplyCircularFilter(cx,cy,0,a,1);
              d->AplyBoxFilter(cx-a,cy-a,cx+a,cy+a,1);
              d->four2();
              p8=BuildFFTimage(d);
              if (p8)
                 {
                  sprintf(str,"%s%04i.bmp",file,t++);
                  p8->SaveBMP(str);
                 }
              delete d;
             }
         }
}

int main(int an,char *av[])
{
 do_anim("w.frq",1);
}
